#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include "CalCons.h"


//main function to all the operation
int main (int argc, char *argv[]) 
{
	//get the default values
	if(argc==1)
	{
		PrintReadme();
		return 0;
	}
	INPUTPARAMETER* pInputParam=new INPUTPARAMETER;

	ParseInputParam(pInputParam,argc,argv);
	ConservationScore(pInputParam);

	//delete input param
	delete pInputParam;

	return 0;
}

void ConservationScore(INPUTPARAMETER* pInputParam)
{
	//get motif width 
	int* pnMotifWidth=new int[pInputParam->nMotifNumber];
	char pchrBuffer[10001];

	//read in motif width
	CString strLine;
	FILE* file=fopen(pInputParam->strMotifWidthfile.m_pString,"r");
	if(file)
	{
		for(int k=0;k<pInputParam->nMotifNumber;k++)
		{
			fgets(pchrBuffer,10000,file);
			pnMotifWidth[k]=atoi(pchrBuffer);
		}
		fclose(file);
	}

	unsigned char* pConsBuffer=new unsigned char[pInputParam->nMaxSeqLen];
	int* pnConsScore=new int[pInputParam->nMaxSeqLen];
	file=fopen(pInputParam->strInputMod.m_pString,"r");

	FILE* output=fopen(pInputParam->strOutput.m_pString,"a+");

	if(file && output)
	{
		int nTab[5];
		while(fgets(pchrBuffer,10000,file))
		{
			if(pchrBuffer[0]=='>')
			{
				strLine=pchrBuffer;
				nTab[0]=strLine.Find('\t');
				nTab[4]=strLine.GetLength();
				int i=0;
				for(i=1;i<4;i++)
					nTab[i]=strLine.Find('\t',nTab[i-1]+1);

				CString strTemp;
				strTemp=strLine.Mid(nTab[0]+1,nTab[1]-nTab[0]-1);
				CString strConsfile;
				strConsfile=pInputParam->strConsPath+"_"+strTemp+"_"+strLine.Mid(1,nTab[0]-1)+".cs";

				//read conservation score file
				FILE* consfile=fopen(strConsfile.m_pString,"rb");
				if(consfile)
				{
					fread(pConsBuffer,1,pInputParam->nMaxSeqLen,consfile);
					fclose(consfile);

					for(i=0;i<pInputParam->nMaxSeqLen;i++) 
						pnConsScore[i]=int(pConsBuffer[i]);
				}
				else //no conservation 
				{
					for(i=0;i<pInputParam->nMaxSeqLen;i++) 
						pnConsScore[i]=0;
				}

				//Get module locations
				int nMid=strLine.Find(',',nTab[1]);
				strTemp=strLine.Mid(nTab[1]+2,nMid-(nTab[1]+2));
				int nModStart=atoi(strTemp.m_pString);
				strTemp=strLine.Mid(nMid+1,nTab[2]-1-(nMid+1));
				int nModEnd=atoi(strTemp.m_pString);

				double dblModCs=0;
				int nPosCount=0;
				for(i=nModStart;i<nModEnd;i++)
				{
					dblModCs+=pnConsScore[i];
					nPosCount++;
				}
				dblModCs/=(255*nPosCount);

				//Get motif locations
				int nRightParen=nTab[3];
				int nLeftParen=strLine.Find('(',nRightParen);
				double dblMotifCs=0;;
				int nMotifBaseCount=0;
				while(nLeftParen>0)
				{
					strTemp=strLine.Mid(nRightParen+2,nLeftParen-(nRightParen+2));
					int k=atoi(strTemp.m_pString)-1;
					nRightParen=strLine.Find(')',nLeftParen);
					strTemp=strLine.Mid(nLeftParen+1,nRightParen-(nLeftParen+1));
					int nMotifStart=atoi(strTemp.m_pString);
					for(i=nMotifStart;i<nMotifStart+pnMotifWidth[k];i++)
					{
						dblMotifCs+=pnConsScore[i];
						nMotifBaseCount++;
					}
					
					nLeftParen=strLine.Find('(',nRightParen);
				}
				dblMotifCs/=(255*nMotifBaseCount);

				//output conservation scores
				fprintf(output,strLine.Left(nTab[1]).m_pString);
				fprintf(output,"\t%f\t%f\n",dblModCs,dblMotifCs);
			}
		}

		fclose(file);
		fclose(output);
	}
	else
	{
		printf("Cannot open module file!\n");
	}

	delete[] pnMotifWidth;
	delete[] pConsBuffer;
	delete[] pnConsScore;
}

//parse input parameters for the program
void ParseInputParam(INPUTPARAMETER* pInputParam, int argc, char* argv[])
{
	for(int i=0;i<argc;i++)
	{
		if(argv[i][0]=='-')
		{
			if(argv[i][1]=='i')
			{
				pInputParam->strInputMod=argv[i+1];
			}
			else if(argv[i][1]=='o')
			{
				pInputParam->strOutput=argv[i+1];
			}
			else if(argv[i][1]=='c')
			{
				pInputParam->strConsPath=argv[i+1];
			}
			else if(argv[i][1]=='w')	//motifwidthfile
			{
				pInputParam->strMotifWidthfile=argv[i+1];
			}
			else if(argv[i][1]=='K')
			{
				pInputParam->nMotifNumber=atoi(argv[i+1]);
			}
			else if(argv[i][1]=='L')
			{
				pInputParam->nMaxSeqLen=atoi(argv[i+1]);
			}
		}
	}
}


void PrintReadme()
{
	printf("\n******************************\n");
	printf("   CisModScan 1.0 \n");
	printf("******************************\n");
	printf("\nUsage: ./CisModScan -i Seqfile (options)\n");
	printf("Seqfile must be in FASTA format and each line in the file should contain no more than 10K characters\n\n");
	printf("Options:\n");
	printf("\t-o\tOutput file (default output.txt)\n");
	printf("\t-r\t1: search for both strands; 0: only search for forward strand (default 1)\n");
	printf("\t-K\tNumber of transcription factors (default 3)\n");
	printf("\t-L\tModule length (default 200)\n");
	printf("\t-M\tInput text files for weight matrices\n");
	printf("\t-x\tExpected module occurrent rate (default 0.0005)\n");
}

