#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include "GibbsDP.h"
#include "StochasticDistribution.h"

#define LOGHALF	-0.6931

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGibbsDP::CGibbsDP()
{
	m_nMotifNumber=1;			//# of motifs: K
	m_nModuleLength=200;		//length of each module: l
	m_nSequenceNumber=0;		//number of sequences
	m_bReverse=true;			//sample backward or not

	m_pnWeightMat=NULL;			//weight matrices for all the motifs
	m_pnMotifWidth=NULL;		//width for each motif
	m_pnSiteCount=NULL;			//number of site for each motif
	m_bReverse=true;			//search reverse strand

	m_pSiteLink=NULL;			//Links for motif site
	m_pModuleLink=NULL;		//Links for modules	

}

CGibbsDP::~CGibbsDP()
{
	FreeMemery();
}

void CGibbsDP::FreeMemery()
{
	if(m_pnMotifWidth!=NULL)
	{
		delete[] m_pnMotifWidth;
		m_pnMotifWidth=NULL;
	}
	if(m_pnSiteCount!=NULL)
	{
		delete[] m_pnSiteCount;
		m_pnSiteCount=NULL;
	}
	if(m_pnWeightMat!=NULL)
	{
		for(int k=0;k<m_nMotifNumber;k++)
		{
			if(m_pnWeightMat[k]!=NULL)
			{
				delete[] m_pnWeightMat[k];
				m_pnWeightMat[k]=NULL;
			}
		}
		delete[] m_pnWeightMat;
		m_pnWeightMat=NULL;
	}
	if(m_pSiteLink!=NULL)
	{
		for(int k=0;k<m_nMotifNumber;k++)
		{
			DelSiteHead(m_pSiteLink[k]);
		}
		delete[] m_pSiteLink;
	}
	if(m_pModuleLink!=NULL)
	{
		DelModHead(m_pModuleLink);
	}
}

//delete sitehead of one motif
void CGibbsDP::DelSiteHead(SITEINFO* pSiteHead)		
{
	if(pSiteHead!=NULL)
	{
		SITEINFO* pCur=pSiteHead;
		SITEINFO* pDel;
		while(pCur)
		{
			pDel=pCur;
			pCur=pCur->pNextSite;
			delete pDel;
		}
	}
}

void CGibbsDP::DelModHead(MODULEINFO* pModHead)		
{
	if(pModHead!=NULL)
	{
		MODULEINFO* pCur=pModHead;
		MODULEINFO* pDel;
		while(pCur)
		{
			pDel=pCur;
			pCur=pCur->pNextModule;
			delete pDel;
		}
	}
}

void CGibbsDP::AddSite(SITEINFO* pSiteHead,SITEINFO* pNewSite)
{
	pNewSite->pNextSite=pSiteHead->pNextSite;
	pSiteHead->pNextSite=pNewSite;
}


void CGibbsDP::ModuleScan()
{
	srand( (unsigned)time( NULL ) );

	//set up paramters
	double** Theta=new double*[m_nMotifNumber];
	Initialize(Theta);


	//definition of parameters
	double r=m_dblr;
	double* q=new double[m_nMotifNumber+1];
	int k=0;
	for(k=0;k<m_nMotifNumber;k++)
		q[k]=0.8/m_nModuleLength;
	q[m_nMotifNumber]=1-double(0.8*m_nMotifNumber)/m_nModuleLength;

	//read in one sequence and then search for module
	m_nModuleCount=0;
	for(k=0;k<m_nMotifNumber;k++) m_pnSiteCount[k]=0;
	int nMaxChar=10000;
	FILE* InputFile=fopen(m_strInputSeq.m_pString,"r");
	CString strLine;
	char    pchrSeqIndex[100];
	m_bFirstSeq=true;
	if(InputFile)
	{

		int i=-1;
		char pcharLine[10001];
		while(fgets(pcharLine,nMaxChar,InputFile))
		{
			if(pcharLine[0]=='>')
			{
				i++;
				if(i>=1)
				{
					DataAugmentation(r,q,Theta);
					m_bFirstSeq=false;
				}
				m_pstrSeqName=pcharLine;
				m_pstrSeqName=m_pstrSeqName.Left(m_pstrSeqName.GetLength()-1);
				sprintf(pchrSeqIndex,"\t%d",i);
				m_pstrSeqName=m_pstrSeqName+pchrSeqIndex;
				m_pstrSequence="";
			}
			else
			{
				strLine=pcharLine;
				//change 'a'->'A'
				int nLength=strLine.GetLength();
				for(int j=0;j<nLength;j++)
				{
					if(strLine[j]>='a' && strLine[j] <='z')
					{
						char ch=strLine[j];
						ch+='A'-'a';
						strLine.SetAt(j,ch);
					}
					else if(strLine[j]<'A' || strLine[j]>'Z')
					{
						nLength=j;
						break;
					}
				}
				strLine=strLine.Left(nLength);
				m_pstrSequence=m_pstrSequence+strLine;
			}
		}
		fclose(InputFile);
	}

	DataAugmentation(r,q,Theta);
	WriteSummary();

	delete[] q;
	for(k=0;k<m_nMotifNumber;k++)
		delete[] Theta[k];
	delete[] Theta;
}


void CGibbsDP::DataAugmentation(double r,double* q, double** Theta)
{
	m_nSeqLength=m_pstrSequence.GetLength();
	BuildBackground();

	//initialize the module and site links

	if(m_pModuleLink!=NULL)
	{
		DelModHead(m_pModuleLink);
	}
	m_pModuleLink=new MODULEINFO;
	m_pModuleLink->pNextModule=NULL;
	m_pModuleLink->nStart=-1;
	m_pModuleLink->dblLogModScore=0;

	int k=0;
	for(k=0;k<m_nMotifNumber;k++)
	{
		DelSiteHead(m_pSiteLink[k]);
	}
	for(k=0;k<m_nMotifNumber;k++)
	{
		m_pSiteLink[k]=new SITEINFO;
		m_pSiteLink[k]->pNextSite=NULL;
	}

	//get sequence lengths and the probability of positions
	int nAvgSeqLength=0;
	int n=0;
	double* pdblLogPrPos=new double[m_nSeqLength*(m_nMotifNumber*2+1)];
	CalLogPrPos(pdblLogPrPos,m_nSeqLength,Theta,n);
	int nMLPropIndex=-1;
//	ApproxSampleCurSeq(r,q,Theta,pdblLogPrPos);
	ApproxScanCurSeq(r,q,Theta,pdblLogPrPos);
	delete[] pdblLogPrPos;

	OutputModules();
	OutputMotifs();
}

void CGibbsDP::Initialize(double** Theta)
{
	//motif width
	m_pnMotifWidth=new int[m_nMotifNumber];


	ReadInputPWM(Theta);



	//weight matrix
	int k=0;
	m_pnWeightMat=new int*[m_nMotifNumber];
	for(k=0;k<m_nMotifNumber;k++)
	{
		m_pnWeightMat[k]=new int[4*m_pnMotifWidth[k]];
	}

	//number of sites for each motif: including background in module
	m_pnSiteCount=new int[m_nMotifNumber+1];

	m_pSiteLink=new SITEINFO*[m_nMotifNumber];
	for(k=0;k<m_nMotifNumber;k++)
		m_pSiteLink[k]=NULL;


}

void CGibbsDP::ReadInputPWM(double** pdblTheta)				//read in the PWMs
{
	char pchrLine[1001];
	CString strLine;
	int nMaxChar=1000;
	FILE* file=fopen(m_strPWMFile.m_pString,"r");
	if(file)
	{
		int k=-1;
		while(fgets(pchrLine,nMaxChar,file))
		{
			if(pchrLine[0]=='>')
			{
				k++;
				strLine=pchrLine;
				strLine=strLine.Right(strLine.GetLength()-1);
				m_pnMotifWidth[k]=atoi(strLine.m_pString);
				pdblTheta[k]=new double[4*m_pnMotifWidth[k]];

				//read in the weight matrix
				for(int i=0;i<m_pnMotifWidth[k];i++)
				{
					fgets(pchrLine,nMaxChar,file);
					strLine=pchrLine;
					int nLength=strLine.GetLength();
					int nStart=0;
					int j=0;
					int nEnd=nStart;
					for(int b=0;b<4;b++)
					{
						for(j=nStart+1;j<nLength;j++)
						{
							if(strLine[j]=='\t' || strLine[j]=='\n')
							{
								nEnd=j;
								break;
							}
						}
						CString strTemp=strLine.Mid(nStart,nEnd-nStart);
						pdblTheta[k][4*i+b]=atof(strTemp.m_pString);
						nStart=nEnd;
					}
				}
			}
		}
		fclose(file);
	}

	//normalize the weight matrices
	for(int k=0;k<m_nMotifNumber;k++)
	{
		for(int i=0;i<m_pnMotifWidth[k];i++)
		{
			double dblRowSum=0;
			int j=0;
			for(j=0;j<4;j++)
				dblRowSum+=pdblTheta[k][4*i+j];
			for(j=0;j<4;j++)
				pdblTheta[k][4*i+j]=(pdblTheta[k][4*i+j]+dblRowSum*0.0025)/(dblRowSum*1.01);
		}
	}
}


void CGibbsDP::BuildBackground()				//build background by the input seq.
{
	int i=0;
	for(i=0;i<20;i++)
		m_dblBackGround[i]=0;
	int nLength=m_nSeqLength;
	CString strDouble;
	for(i=0;i<nLength;i++)
	{
		//calculate the frequenc of A C G T
		switch(m_pstrSequence[i])
		{
		case 'A':
			m_dblBackGround[0]++;
			break;
		case 'T':
			m_dblBackGround[3]++;
			break;
		case 'C':
			m_dblBackGround[1]++;
			break;
		case 'G':
			m_dblBackGround[2]++;
			break;
		}
		//calculate the frequence of AA, AC, AG, AT...
		if(i<nLength-1)
		{
			strDouble=m_pstrSequence.Mid(i,2);
			if(strDouble.Find("N")>-1) continue;
			int nCode=WordCode(strDouble);
			m_dblBackGround[nCode+4]++;
		}
	}
	//Transition Pr of AA AC ...
	for(int j=0;j<4;j++)
		for(i=0;i<4;i++)
			m_dblBackGround[(j+1)*4+i]/=m_dblBackGround[j];
	double dblTotal=0;
	//get the Pr of A C G T
	for(i=0;i<4;i++)
		dblTotal+=m_dblBackGround[i];
	for(i=0;i<4;i++)
		m_dblBackGround[i]/=dblTotal;
}

//calculate the log pr of each position under all the K+1 models
void CGibbsDP::CalLogPrPos(double* pdblLogPrPos,int nLength,double** Theta_n,int n)		//calculate the log probability for each position 
{
	CString strWindow;
	for(int i=0;i<nLength;i++)
	{
		//calculate the motif pr [i-w+1,i]
		for(int k=0;k<m_nMotifNumber;k++)
		{
			if(i<m_pnMotifWidth[k]-1)
			{
				pdblLogPrPos[i*(m_nMotifNumber*2+1)+2*k]=-2000;
				pdblLogPrPos[i*(m_nMotifNumber*2+1)+2*k+1]=-2000;				
			}
			else
			{
				strWindow=m_pstrSequence.Mid(i-m_pnMotifWidth[k]+1,m_pnMotifWidth[k]);
				//check if strWindow contains "N" or "n"
				if(strWindow.Find("N") >-1 || strWindow.Find("n") >-1)
				{
					pdblLogPrPos[i*(m_nMotifNumber*2+1)+2*k]=-2000;
					pdblLogPrPos[i*(m_nMotifNumber*2+1)+2*k+1]=-2000;
					continue;
				}

				pdblLogPrPos[i*(m_nMotifNumber*2+1)+2*k]=CalLogMotifPr(strWindow,Theta_n[k]);
				if(m_bReverse)
					pdblLogPrPos[i*(m_nMotifNumber*2+1)+2*k+1]=CalLogMotifPr(Reverse(strWindow),Theta_n[k]);
				else
					pdblLogPrPos[i*(m_nMotifNumber*2+1)+2*k+1]=-2000;
			}
		}

		//calculate the background transition
		if(i==0)
		{
			//if the current position is 'N' or 'n':
			if(m_pstrSequence[0]=='N' || m_pstrSequence[0]=='n')
				pdblLogPrPos[2*m_nMotifNumber]=-1.386; //log(1/4);
			else
			{
				CString strtemp=m_pstrSequence[0];
				pdblLogPrPos[2*m_nMotifNumber]=log(m_dblBackGround[WordCode(strtemp)]);
			}
		}
		else
		{
			strWindow=m_pstrSequence.Mid(i-1,2);

			if(strWindow.Find("N") >-1 || strWindow.Find("n") >-1)
			{
				pdblLogPrPos[i*(m_nMotifNumber*2+1)+2*m_nMotifNumber]=-1.386;
			}
			else
			{
				int nFirst=WordCode(strWindow[0]);
				int nSecond=WordCode(strWindow[1]);
				pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber]=log(m_dblBackGround[nFirst*4+nSecond+4]);
			}
		}
	}
}

//calculate log Motif probability
double CGibbsDP::CalLogMotifPr(CString strWindow,double* Theta)
{
	double dblLogPrMotif=0;
	int nMotifWidth=strWindow.GetLength();
	int nBase;
	for(int i=0;i<nMotifWidth;i++)
	{
		nBase=WordCode(strWindow[i]);
		dblLogPrMotif+=log(Theta[i*4+nBase]);
	}
	return dblLogPrMotif;
}

//calculate background module probability
double CGibbsDP::CalLogBgPr(int nStart,int nWidth,double* pdblLogPrPos)
{
	double dblLogBgPr=0;
	for(int i=nStart;i<nStart+nWidth;i++)
		dblLogBgPr+=pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
	return dblLogBgPr;
}


void CGibbsDP::ApproxSampleCurSeq(double r_n,double* q_n,double** Theta_n,double* pdblLogPrPos)	
{
	//calculate the log pr of each position under all the K+1 models
	int nLength=m_nSeqLength;
	
	//calculate the sequence probability under the module model
	double* pdblLogPrModuleCurPos=new double[nLength];
	CalModuleLogPr(nLength-1,nLength,pdblLogPrPos,q_n,pdblLogPrModuleCurPos);

	//Calculate module probability and forward summation of Module
	double* pModuleLogPr=new double[nLength];
	double* pSumModuleLogPr=new double[nLength];
	int nModuleLength=m_nModuleLength;
	if(nModuleLength>nLength) nModuleLength=nLength;
	
	double dblSum=0;
	int i=0;
	for(i=0;i<nModuleLength-1;i++)
	{
		pModuleLogPr[i]=pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
		dblSum+=pModuleLogPr[i];//+log(1-r_n);
		pSumModuleLogPr[i]=dblSum;
	}
	for(i=nModuleLength-1;i<nLength;i++)
	{
		if(i>=nModuleLength)
			pModuleLogPr[i]=pdblLogPrModuleCurPos[i]-pdblLogPrModuleCurPos[i-nModuleLength];
		else
			pModuleLogPr[i]=pdblLogPrModuleCurPos[i];
		double dblLogP1=pSumModuleLogPr[i-1]+log(1-r_n)+pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
		double dblLogP2;
		if(i==nModuleLength-1)
			dblLogP2=log(r_n)+pModuleLogPr[i];
		else
			dblLogP2=pSumModuleLogPr[i-nModuleLength]+log(r_n)+pModuleLogPr[i];
		pSumModuleLogPr[i]=LogSum(dblLogP1,dblLogP2);
	}

	//1) sample modules
	SampleModule(pModuleLogPr,pSumModuleLogPr,r_n,q_n,nLength,pdblLogPrPos,pdblLogPrModuleCurPos);

	delete[] pModuleLogPr;
	delete[] pSumModuleLogPr;	
	delete[] pdblLogPrModuleCurPos;
}

//forward summation of the marginal probability of being a module
double CGibbsDP::CalModuleLogPr(int nEnd,int nLength,double* pdblLogPrPos,double* q_n,double* pdblPr)					//calculate the log pr of module
{
	int nMinWidth=m_pnMotifWidth[Minimum(m_pnMotifWidth,m_nMotifNumber)];

	int nStart=nEnd-nLength+1;
	pdblPr[0]=pdblLogPrPos[nStart*(2*m_nMotifNumber+1)+2*m_nMotifNumber]
		+log(q_n[m_nMotifNumber]);
	double dblLogPrM;
	for(int i=nStart+1;i<=nEnd;i++)
	{
		pdblPr[i-nStart]=pdblPr[i-nStart-1]+pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber]+log(q_n[m_nMotifNumber]);
//		if(i-nStart+1>=nMinWidth)
//			pdblPr[i-nStart]+=log(q_n[m_nMotifNumber]);
		if(!m_bReverse)	//only forward chain
		{
			for(int k=0;k<m_nMotifNumber;k++)
			{
				if(i-nStart+1>=m_pnMotifWidth[k])
				{
					dblLogPrM=pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k]+log(q_n[k]);
				}
				if(i-nStart+1>m_pnMotifWidth[k])
				{
					dblLogPrM+=pdblPr[i-nStart-m_pnMotifWidth[k]];						
				}
				if(i-nStart+1>=m_pnMotifWidth[k])
					pdblPr[i-nStart]=LogSum(pdblPr[i-nStart],dblLogPrM);
			}
		}
		else	//both strand
		{
			for(int k=0;k<m_nMotifNumber;k++)
			{
				if(i-nStart+1>=m_pnMotifWidth[k])
				{
					dblLogPrM=
						LogSum(pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k],pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k+1]);
					dblLogPrM+=log(q_n[k])+log(0.5);
				}
				if(i-nStart+1>m_pnMotifWidth[k])
				{
					dblLogPrM+=pdblPr[i-nStart-m_pnMotifWidth[k]];
				}
				if(i-nStart+1>=m_pnMotifWidth[k])
					pdblPr[i-nStart]=LogSum(pdblPr[i-nStart],dblLogPrM);
			}
		}
	}
	//get the minimum width of all the motifs
	double dblLogPrModule=pdblPr[nLength-1];
	return dblLogPrModule;
}

//backward sampling for the modules
void CGibbsDP::SampleModule(double* pModuleLogPr,double* pSumModuleLogPr,double r_n,double* q,
							int nLength,double* pdblLogPrPos,double* pdblLogPrModuleCurPos)
{
	//record the starting positions for the modules
	double dblLogPrBg=0;
	double dblLogPrMd=0;
	int i=nLength-1;

	int nModuleLength=m_nModuleLength;
	if(nModuleLength>nLength) nModuleLength=nLength;

	while(i>=nModuleLength-1)
	{
		if(i==nModuleLength-1)
			dblLogPrMd=pModuleLogPr[i]+log(r_n)-pSumModuleLogPr[i];
		else
			dblLogPrMd=pSumModuleLogPr[i-nModuleLength]+pModuleLogPr[i]+log(r_n)-pSumModuleLogPr[i];

		bool bModule=false;
		if(dblLogPrMd>LOGHALF)
			bModule=true;

		if(bModule)
		{
			m_nModuleCount++;
			CString strStructure=SampleSites(q,pdblLogPrPos,i,nModuleLength);
			i=i-nModuleLength;
			MODULEINFO* pNewModule=new MODULEINFO;
			pNewModule->nStart=i+1;
			pNewModule->dblLogModScore=exp(dblLogPrMd);
			pNewModule->strStucture=strStructure;
			pNewModule->pNextModule=m_pModuleLink->pNextModule;
			m_pModuleLink->pNextModule=pNewModule;
		}
		else
			i--;
	}
}


void CGibbsDP::ApproxScanCurSeq(double r_n,double* q_n,double** Theta_n,double* pdblLogPrPos)	
{
	//calculate the log pr of each position under all the K+1 models
	int nLength=m_nSeqLength;
	
	//calculate the sequence probability under the module model
	double* pdblLogPrModuleCurPos=new double[nLength];
	CalModuleLogPr(nLength-1,nLength,pdblLogPrPos,q_n,pdblLogPrModuleCurPos);

	//Calculate module probability and forward summation of Module
	double* pModuleLogPr=new double[nLength];
	double* pMaxModuleLogPr=new double[nLength];
	bool* pbMaxState=new bool[nLength];		//1:module, 0:background
	int nModuleLength=m_nModuleLength;
	if(nModuleLength>nLength) nModuleLength=nLength;
	
	double dblSum=0;
	int i=0;
	for(i=0;i<nModuleLength-1;i++)
	{
		pModuleLogPr[i]=pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
		dblSum+=pModuleLogPr[i];
		pMaxModuleLogPr[i]=dblSum;
		pbMaxState[i]=false;
	}
	for(i=nModuleLength-1;i<nLength;i++)
	{
		if(i>=nModuleLength)
			pModuleLogPr[i]=pdblLogPrModuleCurPos[i]-pdblLogPrModuleCurPos[i-nModuleLength];
		else
			pModuleLogPr[i]=pdblLogPrModuleCurPos[i];
		double dblLogP1=pMaxModuleLogPr[i-1]+log(1-r_n)+pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
		double dblLogP2;
		if(i==nModuleLength-1)
			dblLogP2=log(r_n)+pModuleLogPr[i];
		else
			dblLogP2=pMaxModuleLogPr[i-nModuleLength]+log(r_n)+pModuleLogPr[i];
		if(dblLogP1>dblLogP2) 
		{
			pMaxModuleLogPr[i]=dblLogP1;
			pbMaxState[i]=false;
		}
		else
		{
			pMaxModuleLogPr[i]=dblLogP2;
			pbMaxState[i]=true;
		}
	}

	//1) sample modules
	MaximizeModule(pModuleLogPr,r_n,q_n,nLength,pdblLogPrPos,pbMaxState);

	delete[] pModuleLogPr;
	delete[] pMaxModuleLogPr;	
	delete[] pdblLogPrModuleCurPos;
	delete[] pbMaxState;
}

void CGibbsDP::MaximizeModule(double* pModuleLogPr,double r_n,double* q,
							int nLength,double* pdblLogPrPos,bool* pbMaxState)
{
	//record the starting positions for the modules
	int i=nLength-1;

	int nModuleLength=m_nModuleLength;
	if(nModuleLength>nLength) nModuleLength=nLength;

	while(i>=nModuleLength-1)
	{
		if(pbMaxState[i])
		{
			m_nModuleCount++;
			CString strStructure=SampleSites(q,pdblLogPrPos,i,nModuleLength);
			if(strStructure.m_pString[0]!='N')
			{
				double dblLogBgPr=CalLogBgPr(i-nModuleLength+1,nModuleLength,pdblLogPrPos);
				MODULEINFO* pNewModule=new MODULEINFO;
				pNewModule->nStart=i-nModuleLength+1;
				pNewModule->dblLogModScore=pModuleLogPr[i]-dblLogBgPr;
				pNewModule->strStucture=strStructure;
				pNewModule->pNextModule=m_pModuleLink->pNextModule;
				m_pModuleLink->pNextModule=pNewModule;
			}
			i=i-nModuleLength;
		}
		else
			i--;
	}
}

CString CGibbsDP::SampleSites(double* q_n,double* pdblLogPrPos,int nEnd, int nModuleLength)
{
	CString strStructure;
	int nMinWidth=m_pnMotifWidth[Minimum(m_pnMotifWidth,m_nMotifNumber)];
	
	//forward summation:
	double* pdblPr=new double[nModuleLength];
	int nStart=nEnd-nModuleLength+1;
	pdblPr[0]=pdblLogPrPos[nStart*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
	//	+log(q_n[m_nMotifNumber]);
	double dblLogPrM;
	int i=0;
	for(i=nStart+1;i<=nEnd;i++)
	{
		pdblPr[i-nStart]=pdblPr[i-nStart-1]+pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
		if(i-nStart+1>=nMinWidth)
			pdblPr[i-nStart]+=log(q_n[m_nMotifNumber]);
		if(!m_bReverse)	//only forward chain
		{
			for(int k=0;k<m_nMotifNumber;k++)
			{
				if(i-nStart+1>=m_pnMotifWidth[k])
				{
					dblLogPrM=pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k]+log(q_n[k]);
				}
				if(i-nStart+1>m_pnMotifWidth[k])
				{
					dblLogPrM+=pdblPr[i-nStart-m_pnMotifWidth[k]];						
				}
				if(i-nStart+1>=m_pnMotifWidth[k])
					pdblPr[i-nStart]=LogSum(pdblPr[i-nStart],dblLogPrM);
			}
		}
		else	//both strand
		{
			for(int k=0;k<m_nMotifNumber;k++)
			{
				if(i-nStart+1>=m_pnMotifWidth[k])
				{
					dblLogPrM=
						LogSum(pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k],pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k+1]);
					dblLogPrM+=log(q_n[k])+log(0.5);
				}
				if(i-nStart+1>m_pnMotifWidth[k])
				{
					dblLogPrM+=pdblPr[i-nStart-m_pnMotifWidth[k]];
				}
				if(i-nStart+1>=m_pnMotifWidth[k])
					pdblPr[i-nStart]=LogSum(pdblPr[i-nStart],dblLogPrM);
			}
		}
	}

	//backward sampling
	i=nModuleLength-1;
	double* pdblMotifPrCdf;
	double pdblMotifPr;

	if(!m_bReverse)		//only forward chain is sampling
	{
		pdblMotifPrCdf=new double[m_nMotifNumber+1];
		while(i>=nMinWidth-1)
		{
			double dblBg=1;
			int k=0;
			for(k=0;k<m_nMotifNumber;k++)
			{
				if(i>=m_pnMotifWidth[k])
					pdblMotifPr=pdblPr[i-m_pnMotifWidth[k]]
					+pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+2*k]+log(q_n[k]);
				else if(i==m_pnMotifWidth[k]-1)
					pdblMotifPr=pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+2*k]+log(q_n[k]);
				else
					pdblMotifPr=-2000+pdblPr[i];
				pdblMotifPr-=pdblPr[i];

				if(pdblMotifPr<-200)
					pdblMotifPr=0;
				else
					pdblMotifPr=exp(pdblMotifPr);
				pdblMotifPrCdf[k]=pdblMotifPr;
				dblBg-=pdblMotifPrCdf[k];
			}
//			double dblBg=pdblPr[i-1]
//					+pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+2*m_nMotifNumber]+log(q_n[k])-pdblPr[i];
			pdblMotifPrCdf[m_nMotifNumber]=dblBg;

			int nMotif=Maximum(pdblMotifPrCdf,m_nMotifNumber+1);

			if(nMotif<m_nMotifNumber)
			{
				m_pnSiteCount[nMotif]++;
				SITEINFO* pNewSite=new SITEINFO;
				pNewSite->bForward=true;
				pNewSite->nPos=nStart+i-m_pnMotifWidth[nMotif]+1;
				pNewSite->strSite=m_pstrSequence.Mid(pNewSite->nPos,m_pnMotifWidth[nMotif]);
				pNewSite->dblScore=pdblMotifPrCdf[nMotif];
				pNewSite->pNextSite=NULL;
				AddSite(m_pSiteLink[nMotif],pNewSite);
				char pchrCurSite[100];
				sprintf(pchrCurSite,"+%d(%d)",nMotif+1,pNewSite->nPos);
				CString strCurSite;
				strCurSite=pchrCurSite;
				strStructure=strCurSite+strStructure;
				i=i-m_pnMotifWidth[nMotif];
			}
			else
				i--;
		}
	}
	else	//sampling both chain
	{
		pdblMotifPrCdf=new double[m_nMotifNumber*2+1];
		while(i>=nMinWidth-1)
		{
			int k=0;
			double dblBg=1;
			for(k=0;k<2*m_nMotifNumber;k++)
			{
				if(i>=m_pnMotifWidth[k/2])
					pdblMotifPr=pdblPr[i-m_pnMotifWidth[k/2]]
					+pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+k]+log(q_n[k/2])+log(0.5);
				else if(i==m_pnMotifWidth[k/2]-1)
					pdblMotifPr=pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+k]+log(q_n[k/2])+log(0.5);
				else
					pdblMotifPr=-2000+pdblPr[i];
				pdblMotifPr-=pdblPr[i];

				if(pdblMotifPr<-200)
					pdblMotifPr=0;
				else
					pdblMotifPr=exp(pdblMotifPr);
				pdblMotifPrCdf[k]=pdblMotifPr;
				dblBg-=pdblMotifPrCdf[k];
			}
//			double dblBg=pdblPr[i-1]
//					+pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+2*m_nMotifNumber]+log(q_n[m_nMotifNumber])-pdblPr[i];
			pdblMotifPrCdf[2*m_nMotifNumber]=dblBg;

			int nMotif=Maximum(pdblMotifPrCdf,2*m_nMotifNumber+1);

			if(nMotif<2*m_nMotifNumber)
			{
				m_pnSiteCount[nMotif/2]++;
				SITEINFO* pNewSite=new SITEINFO;
				if(nMotif%2==0)
					pNewSite->bForward=true;
				else
					pNewSite->bForward=false;
				pNewSite->nPos=nStart+i-m_pnMotifWidth[nMotif/2]+1;
				if(pNewSite->bForward)
					pNewSite->strSite=m_pstrSequence.Mid(pNewSite->nPos,m_pnMotifWidth[nMotif/2]);
				else
					pNewSite->strSite=Reverse(m_pstrSequence.Mid(pNewSite->nPos,m_pnMotifWidth[nMotif/2]));
				pNewSite->dblScore=pdblMotifPrCdf[nMotif];
				pNewSite->pNextSite=NULL;
				AddSite(m_pSiteLink[nMotif/2],pNewSite);

				char pchrCurSite[100];
				if(pNewSite->bForward)
					sprintf(pchrCurSite,"+%d(%d)",nMotif/2+1,pNewSite->nPos);
				else
					sprintf(pchrCurSite,"-%d(%d)",nMotif/2+1,pNewSite->nPos);
				CString strCurSite;
				strCurSite=pchrCurSite;
				strStructure=strCurSite+strStructure;

				i=i-m_pnMotifWidth[nMotif/2];
			}
			else
				i--;
		}
	}

	delete[] pdblMotifPrCdf;
	delete[] pdblPr;
	if(strStructure.m_nstrLength==0)	//no motif sites found
	{
		strStructure="NULL";
	}
	return strStructure;
}

//output modules for one sequence
void CGibbsDP::OutputModules()	
{
	CString strModuleFile;
	strModuleFile=m_strOutput+"_module.txt";
	FILE* file=fopen(strModuleFile.m_pString,"a+");
	if(file)
	{
		if(m_bFirstSeq)	fprintf(file,m_strInformation.m_pString);
		MODULEINFO* pCurMod=m_pModuleLink->pNextModule;
		while(pCurMod)
		{
			fprintf(file,m_pstrSeqName.m_pString);
			fprintf(file,"\t[%d,%d]\t%.6lf\t",pCurMod->nStart,
				pCurMod->nStart+m_nModuleLength,pCurMod->dblLogModScore);
			fprintf(file,pCurMod->strStucture.m_pString);
			fprintf(file,"\n");
			pCurMod=pCurMod->pNextModule;
		}
		fclose(file);
	}
}

//output motifs		
void CGibbsDP::OutputMotifs()
{
	for(int k=0;k<m_nMotifNumber;k++)
	{
		char chrtemp[50];
		sprintf(chrtemp,"_motif%d.txt",k+1);
		CString strMotifFile;
		strMotifFile=m_strOutput+chrtemp;
		FILE* file=fopen(strMotifFile.m_pString,"a+");
		if(file)
		{
			if(m_bFirstSeq)	fprintf(file,"\n\nmotif %d\n\n",k+1);
			SITEINFO* pCurSite=m_pSiteLink[k]->pNextSite;
			while(pCurSite)
			{
				fprintf(file,m_pstrSeqName.m_pString);
				if(pCurSite->bForward)
					fprintf(file,"\tf\t");
				else
					fprintf(file,"\tr\t");
				fprintf(file,"%d\t%.6lf\n",pCurSite->nPos,pCurSite->dblScore);
				fprintf(file,pCurSite->strSite.m_pString);
				fprintf(file,"\n");
				pCurSite=pCurSite->pNextSite;
			}
			fclose(file);
		}
	}
}

//write summary statistics
void CGibbsDP::WriteSummary()			
{
	CString strModuleFile;
	strModuleFile=m_strOutput+"_module.txt";
	FILE* file=fopen(strModuleFile.m_pString,"a+");
	if(file)
	{
		fprintf(file,"\n# modules = %d\n",m_nModuleCount);
		fclose(file);
	}

	for(int k=0;k<m_nMotifNumber;k++)
	{
		char chrtemp[50];
		sprintf(chrtemp,"_motif%d.txt",k+1);
		CString strMotifFile;
		strMotifFile=m_strOutput+chrtemp;
		FILE* file=fopen(strMotifFile.m_pString,"a+");
		if(file)
			fprintf(file,"\n # of sites for motif %d = %d\n",k+1,m_pnSiteCount[k]);
		fclose(file);
	}
}


/*-------------friend function-----------------------*/
int intpow(int nBase,int nPower)	//calculate interger power
{
	int nResult=1;
	for(int i=0;i<nPower;i++)
	{
		nResult*=nBase;
	}
	return nResult;
}

CString Reverse(CString strSeq)					//reverse the sequence
{
	CString strReverse=strSeq;
	int		nLength=strSeq.GetLength();
	for(int i=0;i<nLength;i++)
	{
		switch(strSeq[i])
		{
		case 'A':
			strReverse.SetAt(nLength-1-i,'T');
			break;
		case 'T':
			strReverse.SetAt(nLength-1-i,'A');
			break;
		case 'G':
			strReverse.SetAt(nLength-1-i,'C');
			break;
		case 'C':
			strReverse.SetAt(nLength-1-i,'G');
			break;
		case 'S':
			strReverse.SetAt(nLength-1-i,'S');
			break;
		case 'R':
			strReverse.SetAt(nLength-1-i,'Y');
			break;
		case 'Y':
			strReverse.SetAt(nLength-1-i,'R');
			break;
		case 'W':
			strReverse.SetAt(nLength-1-i,'W');
			break;
		default:
			strReverse.SetAt(nLength-1-i,strSeq[i]);
		}
	}
	return strReverse;
}

//code the word
int WordCode(CString strWord)
{
	int nLength=strWord.GetLength();
	int nCode=0;
	for(int i=0;i<nLength;i++)
	{
		switch(strWord[i])
		{
		case 'A':
			nCode+=0;
			break;
		case 'C':
			nCode+=1*intpow(4,nLength-i-1);
			break;
		case 'G':
			nCode+=2*intpow(4,nLength-i-1);
			break;
		case 'T':
			nCode+=3*intpow(4,nLength-i-1);
			break;
		}
	}
	return nCode;
}

int Sum(int* pVector,int nSize)
{
	int dblSum=0;
	for(int d=0;d<nSize;d++)
	{
		dblSum+=pVector[d];
	}
	return dblSum;
}

double logGamma(int nX)				//log-gamma function of integer input
{
	if(nX==1) return 0;
	double nY=0;
	for(int j=nX-1;j>1;j--)
		nY+=log(j);
	return nY;
}

int	Minimum(int* pVector,int nSize)
{
	int nMin=0;
	int nMinValue=pVector[0];
	for(int d=1;d<nSize;d++)
	{
		if(pVector[d]<nMinValue)
		{
			nMinValue=pVector[d];
			nMin=d;
		}
	}
	return nMin;
}

int	Maximum(double* pVector,int nSize)
{
	int nMax=0;
	double dblMax=pVector[0];
	for(int d=1;d<nSize;d++)
	{
		if(pVector[d]>dblMax)
		{
			dblMax=pVector[d];
			nMax=d;
		}
	}
	return nMax;
}
