////////////////////////////////////////////////////////////////
//class of the computing core of CisModule project
#include "MyString.h"

struct SITEINFO		//collection of sites for one motif
{
	CString strSite;
	int		nPos;
	bool	bForward;
	double		dblScore;
	SITEINFO* pNextSite;
};

struct MODULEINFO	//information for the modules on one sequence
{
	CString strStucture;	//the module structure
	int		nStart;	//where are they
	double  dblLogModScore;	//log Module scores
	MODULEINFO* pNextModule;	//point to next module
};


class CGibbsDP  
{
public:
	double			m_dblr;				//input value for r
	int				m_nMotifNumber;		//# of motifs: K
	int				m_nModuleLength;	//length of each module: l
	int				m_nModuleCount;		//total number of modules
	int**			m_pnWeightMat;		//weight matrices for all the motifs
	int*			m_pnMotifWidth;		//width for each motif
	int*			m_pnSiteCount;		//number of site for each motif
	SITEINFO**		m_pSiteLink;		//Links for motif site
	MODULEINFO*		m_pModuleLink;		//Links for modules	

	CString		m_pstrSequence;					//sets of input sequences
	CString		m_pstrSeqName;					//names of the sequences ">****"
	CString		m_strPWMFile;					//files for input PWMs
	int			m_nSequenceNumber;				//number of sequences
	CString		m_strOutput;					//output file
	CString		m_strInputSeq;					//input sequence file
	int			m_nSeqLength;					//current sequence length
	double		m_dblBackGround[20];			//background transition probability

	bool			m_bFirstSeq;			//first sequence
	bool			m_bReverse;				//sample backward or not
	CString			m_strInformation;

public:
	CGibbsDP();
	virtual ~CGibbsDP();

	void FreeMemery();					//free all the memory

	void ReadInputPWM(double**);				//read in the PWMs
	void Initialize(double**);					//initialize member variables
	void BuildBackground();				//build background by the input seq.
	double CalLogBgPr(int,int,double*);			//Calculate log background probabilities
	double CalLogMotifPr(CString,double* ); //calculate the log pr of motifs

	int  CalWeightMat(SITEINFO**,int*,int);				//calculate marginal weight matrix
	int  CountModule(MODULEINFO*);		//count the number of modules in the seq set
	void AddSite(SITEINFO*,SITEINFO*);	//add the site in current sequence to the sitelink
	void DelSiteHead(SITEINFO*);		//delete sitehead of one motif
	void DelModHead(MODULEINFO* pModHead);	//delete the module head
	void CalLogPrPos(double*,int,double**,int);		//calculate the log probability for each position 
	void ApproxSampleCurSeq(double,double*,double**,double*);		//sample the current sequence approximately
	double CalModuleLogPr(int,int,double*,double*,double*);		//calculate forward summation
	void SampleModule(double*,double*,double,double*,int,double*,double*);			//sample module on current sequence
	CString SampleSites(double*,double*,int,int);							//sample sites given module
	void ApproxScanCurSeq(double,double*,double**,double*);		//scan the current sequence approximately: FB to find maximum missing data
	void MaximizeModule(double*,double,double*,int,double*,bool*);			//find optimal modules on current sequence

	void DataAugmentation(double,double*,double**);				//sampling both parameters and module\site
	void ModuleScan();						//module scan one sequence by one sequence
	void OutputModules();				//output modules for one sequence
	void OutputMotifs();					//output motifs
	void WriteSummary();					//write summary statistics
	


	//*************friend function
	friend int		Maximum(double*,int);		//find the index for the maximum of a vector
	friend int		Minimum(int*,int);		//minimal of the vector
	friend int Sum(int*,int);				//calculate the sum of the vectors
	friend double Maximum(double,double);		//get the maximum of the two numbers
	friend int		WordCode(CString);			//code the word
	friend CString	DeCodeWord(int,int);		//decode the word
	friend CString Reverse(CString);					//reverse the sequence
	friend int intpow(int,int);						//calculate int power
	friend double LogSum(double,double);		//calculate log(exp(x1)+exp(x2));
	friend double logGamma(int);				//calculate log gamma function
};

